@@ -114,7 +114,7 @@ struct SideBar : public FullWrtStruct::ZoneHeader {
   {
   }
   //! the position (in point)
-  Box2f m_box;
+  MWAWBox2f m_box;
   //! the page
   int m_page;
 };
@@ -260,13 +260,17 @@ protected:
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*type*/)
 {
   if (!listener.get()) {
-    MWAW_DEBUG_MSG(("SubDocument::parse: no listener\n"));
+    MWAW_DEBUG_MSG(("FullWrtParserInternall::SubDocument::parse: no listener\n"));
+    return;
+  }
+  FullWrtParser *parser=dynamic_cast<FullWrtParser *>(m_parser);
+  if (!parser) {
+    MWAW_DEBUG_MSG(("FullWrtParserInternall::SubDocument::parse: no parser\n"));
     return;
   }
-  assert(m_parser);
 
   long pos = m_input->tell();
-  static_cast<FullWrtParser *>(m_parser)->send(m_id);
+  parser->send(m_id);
   m_input->seek(pos, librevenge::RVNG_SEEK_SET);
 }
 }
@@ -319,10 +323,10 @@ void FullWrtParser::newPage(int number)
   }
 }
 
-Vec2f FullWrtParser::getPageLeftTop() const
+MWAWVec2f FullWrtParser::getPageLeftTop() const
 {
-  return Vec2f(float(getPageSpan().getMarginLeft()),
-               float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
+  return MWAWVec2f(float(getPageSpan().getMarginLeft()),
+                   float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
 }
 
 bool FullWrtParser::getBorder(int bId, FullWrtStruct::Border &border) const
@@ -351,7 +355,7 @@ void FullWrtParser::sendGraphic(int id)
     FullWrtParserInternal::DocZoneStruct const &data =
       m_state->m_docZoneList[size_t(id)];
     if (data.m_type != 0x15) {
-      MWAW_DEBUG_MSG(("FullWrtParser::sendGraphic: call for zone[%x]\n", data.m_type));
+      MWAW_DEBUG_MSG(("FullWrtParser::sendGraphic: call for zone[%x]\n", (unsigned int)data.m_type));
     }
   }
   m_graphParser->sendGraphic(m_state->getFileZoneId(id));
@@ -371,9 +375,7 @@ bool FullWrtParser::send(int fileId, MWAWColor fontColor)
 ////////////////////////////////////////////////////////////
 void FullWrtParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  assert(getInput().get() != 0);
-
-  if (!checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
 #ifdef DEBUG
@@ -1050,33 +1052,30 @@ bool FullWrtParser::readPrintInfo(FullWrtStruct::EntryPtr zone)
   // print info
   libmwaw::PrinterInfo info;
   if (!info.read(input)) {
-    if (sz == 0x78) {
-      // the size is ok, so let try to continue
-      asciiFile.addPos(pos);
-      asciiFile.addNote("Entries(PrintInfo):##");
-      input->seek(endPos, librevenge::RVNG_SEEK_SET);
-      MWAW_DEBUG_MSG(("FullWrtParser::readPrintInfo: can not read print info, continue\n"));
-      return true;
-    }
-    return false;
+    // we know that the size is ok, so let try to continue
+    asciiFile.addPos(pos);
+    asciiFile.addNote("Entries(PrintInfo):##");
+    input->seek(endPos, librevenge::RVNG_SEEK_SET);
+    MWAW_DEBUG_MSG(("FullWrtParser::readPrintInfo: can not read print info, continue\n"));
+    return true;
   }
   f << "Entries(PrintInfo):"<< info;
 
-  Vec2i paperSize = info.paper().size();
-  Vec2i pageSize = info.page().size();
+  MWAWVec2i paperSize = info.paper().size();
+  MWAWVec2i pageSize = info.page().size();
   if (pageSize.x() <= 0 || pageSize.y() <= 0 ||
       paperSize.x() <= 0 || paperSize.y() <= 0) return false;
 
   if (!m_state->m_pageSpanSet) {
     // define margin from print info
-    Vec2i lTopMargin= -1 * info.paper().pos(0);
-    Vec2i rBotMargin=info.paper().size() - info.page().size();
+    MWAWVec2i lTopMargin= -1 * info.paper().pos(0);
+    MWAWVec2i rBotMargin=info.paper().size() - info.page().size();
 
     // move margin left | top
     int decalX = lTopMargin.x() > 14 ? lTopMargin.x()-14 : 0;
     int decalY = lTopMargin.y() > 14 ? lTopMargin.y()-14 : 0;
-    lTopMargin -= Vec2i(decalX, decalY);
-    rBotMargin += Vec2i(decalX, decalY);
+    lTopMargin -= MWAWVec2i(decalX, decalY);
+    rBotMargin += MWAWVec2i(decalX, decalY);
 
     // decrease right | bottom
     int rightMarg = rBotMargin.x() -50;
@@ -1921,7 +1920,7 @@ void FullWrtParser::sendReference(int id)
     return;
   }
   if (m_state->m_docZoneList[size_t(id)].m_type != 0x1a) {
-    MWAW_DEBUG_MSG(("FullWrtParser::sendReference: find unexpected type for fieldDataRedirect=%x\n", m_state->m_docZoneList[size_t(id)].m_type));
+    MWAW_DEBUG_MSG(("FullWrtParser::sendReference: find unexpected type for fieldDataRedirect=%x\n", (unsigned int) m_state->m_docZoneList[size_t(id)].m_type));
     return;
   }
   if (m_state->m_referenceRedirectMap.find(id) == m_state->m_referenceRedirectMap.end())
@@ -1951,7 +1950,7 @@ void FullWrtParser::sendVariable(int id)
     return;
   }
   if (m_state->m_docZoneList[size_t(id)].m_type != 0x1e) {
-    MWAW_DEBUG_MSG(("FullWrtParser::sendVariable: find unexpected type for fieldDataRedirect=%x\n", m_state->m_docZoneList[size_t(id)].m_type));
+    MWAW_DEBUG_MSG(("FullWrtParser::sendVariable: find unexpected type for fieldDataRedirect=%x\n", (unsigned int)m_state->m_docZoneList[size_t(id)].m_type));
     return;
   }
   if (m_state->m_variableRedirectMap.find(id) == m_state->m_variableRedirectMap.end()) {
@@ -1979,7 +1978,7 @@ void FullWrtParser::sendVariable(int id)
     }
   }
   else {
-    MWAW_DEBUG_MSG(("FullWrtParser::sendVariable: find unexpected redirection type[%x] for variable %d\n", data.m_type, id));
+    MWAW_DEBUG_MSG(("FullWrtParser::sendVariable: find unexpected redirection type[%x] for variable %d\n", (unsigned int) data.m_type, id));
   }
 }
 
@@ -1997,7 +1996,7 @@ void FullWrtParser::sendText(int id, libmwaw::SubDocumentType type, MWAWNote::Ty
     if (type==libmwaw::DOC_NOTE && (docType==0xc|| docType==0xd)) ;
     else if (type == libmwaw::DOC_COMMENT_ANNOTATION && docType == 0xb) ;
     else {
-      MWAW_DEBUG_MSG(("FullWrtParser::sendText: call with %d[%x]\n", int(type),docType));
+      MWAW_DEBUG_MSG(("FullWrtParser::sendText: call with %d[%x]\n", int(type),(unsigned int)docType));
     }
   }
   else {
